@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2015 Eric Lafortune @ GuardSquare
+ * Copyright (c) 2002-2016 Eric Lafortune @ GuardSquare
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -79,35 +79,35 @@ implements   ClassVisitor,
         dataOutput.writeShort(ClassUtil.internalMajorClassVersion(programClass.u4version));
 
         // Write the constant pool.
-        dataOutput.writeShort(programClass.u2constantPoolCount);
+        dataOutput.writeUnsignedShort(programClass.u2constantPoolCount);
 
         programClass.constantPoolEntriesAccept(this);
 
         // Write the general class information.
-        dataOutput.writeShort(programClass.u2accessFlags);
-        dataOutput.writeShort(programClass.u2thisClass);
-        dataOutput.writeShort(programClass.u2superClass);
+        dataOutput.writeUnsignedShort(programClass.u2accessFlags);
+        dataOutput.writeUnsignedShort(programClass.u2thisClass);
+        dataOutput.writeUnsignedShort(programClass.u2superClass);
 
         // Write the interfaces.
-        dataOutput.writeShort(programClass.u2interfacesCount);
+        dataOutput.writeUnsignedShort(programClass.u2interfacesCount);
 
         for (int index = 0; index < programClass.u2interfacesCount; index++)
         {
-            dataOutput.writeShort(programClass.u2interfaces[index]);
+            dataOutput.writeUnsignedShort(programClass.u2interfaces[index]);
         }
 
         // Write the fields.
-        dataOutput.writeShort(programClass.u2fieldsCount);
+        dataOutput.writeUnsignedShort(programClass.u2fieldsCount);
 
         programClass.fieldsAccept(this);
 
         // Write the methods.
-        dataOutput.writeShort(programClass.u2methodsCount);
+        dataOutput.writeUnsignedShort(programClass.u2methodsCount);
 
         programClass.methodsAccept(this);
 
         // Write the class attributes.
-        dataOutput.writeShort(programClass.u2attributesCount);
+        dataOutput.writeUnsignedShort(programClass.u2attributesCount);
 
         programClass.attributesAccept(this);
     }
@@ -123,12 +123,12 @@ implements   ClassVisitor,
     public void visitProgramField(ProgramClass programClass, ProgramField programField)
     {
         // Write the general field information.
-        dataOutput.writeShort(programField.u2accessFlags);
-        dataOutput.writeShort(programField.u2nameIndex);
-        dataOutput.writeShort(programField.u2descriptorIndex);
+        dataOutput.writeUnsignedShort(programField.u2accessFlags);
+        dataOutput.writeUnsignedShort(programField.u2nameIndex);
+        dataOutput.writeUnsignedShort(programField.u2descriptorIndex);
 
         // Write the field attributes.
-        dataOutput.writeShort(programField.u2attributesCount);
+        dataOutput.writeUnsignedShort(programField.u2attributesCount);
 
         programField.attributesAccept(programClass, this);
     }
@@ -137,12 +137,12 @@ implements   ClassVisitor,
     public void visitProgramMethod(ProgramClass programClass, ProgramMethod programMethod)
     {
         // Write the general method information.
-        dataOutput.writeShort(programMethod.u2accessFlags);
-        dataOutput.writeShort(programMethod.u2nameIndex);
-        dataOutput.writeShort(programMethod.u2descriptorIndex);
+        dataOutput.writeUnsignedShort(programMethod.u2accessFlags);
+        dataOutput.writeUnsignedShort(programMethod.u2nameIndex);
+        dataOutput.writeUnsignedShort(programMethod.u2descriptorIndex);
 
         // Write the method attributes.
-        dataOutput.writeShort(programMethod.u2attributesCount);
+        dataOutput.writeUnsignedShort(programMethod.u2attributesCount);
 
         programMethod.attributesAccept(programClass, this);
     }
@@ -197,7 +197,7 @@ implements   ClassVisitor,
 
         public void visitStringConstant(Clazz clazz, StringConstant stringConstant)
         {
-            dataOutput.writeShort(stringConstant.u2stringIndex);
+            dataOutput.writeUnsignedShort(stringConstant.u2stringIndex);
         }
 
 
@@ -205,48 +205,48 @@ implements   ClassVisitor,
         {
             byte[] bytes = utf8Constant.getBytes();
 
-            dataOutput.writeShort(bytes.length);
+            dataOutput.writeUnsignedShort(bytes.length);
             dataOutput.write(bytes);
         }
 
 
         public void visitInvokeDynamicConstant(Clazz clazz, InvokeDynamicConstant invokeDynamicConstant)
         {
-            dataOutput.writeShort(invokeDynamicConstant.u2bootstrapMethodAttributeIndex);
-            dataOutput.writeShort(invokeDynamicConstant.u2nameAndTypeIndex);
+            dataOutput.writeUnsignedShort(invokeDynamicConstant.u2bootstrapMethodAttributeIndex);
+            dataOutput.writeUnsignedShort(invokeDynamicConstant.u2nameAndTypeIndex);
         }
 
 
         public void visitMethodHandleConstant(Clazz clazz, MethodHandleConstant methodHandleConstant)
         {
             dataOutput.writeByte(methodHandleConstant.u1referenceKind);
-            dataOutput.writeShort(methodHandleConstant.u2referenceIndex);
+            dataOutput.writeUnsignedShort(methodHandleConstant.u2referenceIndex);
         }
 
 
         public void visitAnyRefConstant(Clazz clazz, RefConstant refConstant)
         {
-            dataOutput.writeShort(refConstant.u2classIndex);
-            dataOutput.writeShort(refConstant.u2nameAndTypeIndex);
+            dataOutput.writeUnsignedShort(refConstant.u2classIndex);
+            dataOutput.writeUnsignedShort(refConstant.u2nameAndTypeIndex);
         }
 
 
         public void visitClassConstant(Clazz clazz, ClassConstant classConstant)
         {
-            dataOutput.writeShort(classConstant.u2nameIndex);
+            dataOutput.writeUnsignedShort(classConstant.u2nameIndex);
         }
 
 
         public void visitMethodTypeConstant(Clazz clazz, MethodTypeConstant methodTypeConstant)
         {
-            dataOutput.writeShort(methodTypeConstant.u2descriptorIndex);
+            dataOutput.writeUnsignedShort(methodTypeConstant.u2descriptorIndex);
         }
 
 
         public void visitNameAndTypeConstant(Clazz clazz, NameAndTypeConstant nameAndTypeConstant)
         {
-            dataOutput.writeShort(nameAndTypeConstant.u2nameIndex);
-            dataOutput.writeShort(nameAndTypeConstant.u2descriptorIndex);
+            dataOutput.writeUnsignedShort(nameAndTypeConstant.u2nameIndex);
+            dataOutput.writeUnsignedShort(nameAndTypeConstant.u2descriptorIndex);
         }
     }
 
@@ -256,7 +256,7 @@ implements   ClassVisitor,
     public void visitAnyAttribute(Clazz clazz, Attribute attribute)
     {
         // Write the attribute name index.
-        dataOutput.writeShort(attribute.u2attributeNameIndex);
+        dataOutput.writeUnsignedShort(attribute.u2attributeNameIndex);
 
         // We'll write the attribute body into an array first, so we can
         // automatically figure out its length.
@@ -313,7 +313,7 @@ implements   ClassVisitor,
         public void visitBootstrapMethodsAttribute(Clazz clazz, BootstrapMethodsAttribute bootstrapMethodsAttribute)
         {
             // Write the bootstrap methods.
-            dataOutput.writeShort(bootstrapMethodsAttribute.u2bootstrapMethodsCount);
+            dataOutput.writeUnsignedShort(bootstrapMethodsAttribute.u2bootstrapMethodsCount);
 
             bootstrapMethodsAttribute.bootstrapMethodEntriesAccept(clazz, this);
         }
@@ -321,20 +321,20 @@ implements   ClassVisitor,
 
         public void visitSourceFileAttribute(Clazz clazz, SourceFileAttribute sourceFileAttribute)
         {
-            dataOutput.writeShort(sourceFileAttribute.u2sourceFileIndex);
+            dataOutput.writeUnsignedShort(sourceFileAttribute.u2sourceFileIndex);
         }
 
 
         public void visitSourceDirAttribute(Clazz clazz, SourceDirAttribute sourceDirAttribute)
         {
-            dataOutput.writeShort(sourceDirAttribute.u2sourceDirIndex);
+            dataOutput.writeUnsignedShort(sourceDirAttribute.u2sourceDirIndex);
         }
 
 
         public void visitInnerClassesAttribute(Clazz clazz, InnerClassesAttribute innerClassesAttribute)
         {
             // Write the inner classes.
-            dataOutput.writeShort(innerClassesAttribute.u2classesCount);
+            dataOutput.writeUnsignedShort(innerClassesAttribute.u2classesCount);
 
             innerClassesAttribute.innerClassEntriesAccept(clazz, this);
         }
@@ -342,8 +342,8 @@ implements   ClassVisitor,
 
         public void visitEnclosingMethodAttribute(Clazz clazz, EnclosingMethodAttribute enclosingMethodAttribute)
         {
-            dataOutput.writeShort(enclosingMethodAttribute.u2classIndex);
-            dataOutput.writeShort(enclosingMethodAttribute.u2nameAndTypeIndex);
+            dataOutput.writeUnsignedShort(enclosingMethodAttribute.u2classIndex);
+            dataOutput.writeUnsignedShort(enclosingMethodAttribute.u2nameAndTypeIndex);
         }
 
 
@@ -361,13 +361,13 @@ implements   ClassVisitor,
 
         public void visitSignatureAttribute(Clazz clazz, SignatureAttribute signatureAttribute)
         {
-            dataOutput.writeShort(signatureAttribute.u2signatureIndex);
+            dataOutput.writeUnsignedShort(signatureAttribute.u2signatureIndex);
         }
 
 
         public void visitConstantValueAttribute(Clazz clazz, Field field, ConstantValueAttribute constantValueAttribute)
         {
-            dataOutput.writeShort(constantValueAttribute.u2constantValueIndex);
+            dataOutput.writeUnsignedShort(constantValueAttribute.u2constantValueIndex);
         }
 
 
@@ -383,11 +383,11 @@ implements   ClassVisitor,
         public void visitExceptionsAttribute(Clazz clazz, Method method, ExceptionsAttribute exceptionsAttribute)
         {
             // Write the exceptions.
-            dataOutput.writeShort(exceptionsAttribute.u2exceptionIndexTableLength);
+            dataOutput.writeUnsignedShort(exceptionsAttribute.u2exceptionIndexTableLength);
 
             for (int index = 0; index < exceptionsAttribute.u2exceptionIndexTableLength; index++)
             {
-                dataOutput.writeShort(exceptionsAttribute.u2exceptionIndexTable[index]);
+                dataOutput.writeUnsignedShort(exceptionsAttribute.u2exceptionIndexTable[index]);
             }
         }
 
@@ -395,8 +395,8 @@ implements   ClassVisitor,
         public void visitCodeAttribute(Clazz clazz, Method method, CodeAttribute codeAttribute)
         {
             // Write the stack size and local variable frame size.
-            dataOutput.writeShort(codeAttribute.u2maxStack);
-            dataOutput.writeShort(codeAttribute.u2maxLocals);
+            dataOutput.writeUnsignedShort(codeAttribute.u2maxStack);
+            dataOutput.writeUnsignedShort(codeAttribute.u2maxLocals);
 
             // Write the byte code.
             dataOutput.writeInt(codeAttribute.u4codeLength);
@@ -404,12 +404,12 @@ implements   ClassVisitor,
             dataOutput.write(codeAttribute.code, 0, codeAttribute.u4codeLength);
 
             // Write the exceptions.
-            dataOutput.writeShort(codeAttribute.u2exceptionTableLength);
+            dataOutput.writeUnsignedShort(codeAttribute.u2exceptionTableLength);
 
             codeAttribute.exceptionsAccept(clazz, method, this);
 
             // Write the code attributes.
-            dataOutput.writeShort(codeAttribute.u2attributesCount);
+            dataOutput.writeUnsignedShort(codeAttribute.u2attributesCount);
 
             codeAttribute.attributesAccept(clazz, method, ProgramClassWriter.this);
         }
@@ -418,7 +418,7 @@ implements   ClassVisitor,
         public void visitStackMapAttribute(Clazz clazz, Method method, CodeAttribute codeAttribute, StackMapAttribute stackMapAttribute)
         {
             // Write the stack map frames (only full frames, without tag).
-            dataOutput.writeShort(stackMapAttribute.u2stackMapFramesCount);
+            dataOutput.writeUnsignedShort(stackMapAttribute.u2stackMapFramesCount);
 
             stackMapAttribute.stackMapFramesAccept(clazz, method, codeAttribute, stackMapFrameBodyWriter);
         }
@@ -427,7 +427,7 @@ implements   ClassVisitor,
         public void visitStackMapTableAttribute(Clazz clazz, Method method, CodeAttribute codeAttribute, StackMapTableAttribute stackMapTableAttribute)
         {
             // Write the stack map frames.
-            dataOutput.writeShort(stackMapTableAttribute.u2stackMapFramesCount);
+            dataOutput.writeUnsignedShort(stackMapTableAttribute.u2stackMapFramesCount);
 
             stackMapTableAttribute.stackMapFramesAccept(clazz, method, codeAttribute, this);
         }
@@ -436,7 +436,7 @@ implements   ClassVisitor,
         public void visitLineNumberTableAttribute(Clazz clazz, Method method, CodeAttribute codeAttribute, LineNumberTableAttribute lineNumberTableAttribute)
         {
             // Write the line numbers.
-            dataOutput.writeShort(lineNumberTableAttribute.u2lineNumberTableLength);
+            dataOutput.writeUnsignedShort(lineNumberTableAttribute.u2lineNumberTableLength);
 
             lineNumberTableAttribute.lineNumbersAccept(clazz, method, codeAttribute, this);
         }
@@ -445,7 +445,7 @@ implements   ClassVisitor,
         public void visitLocalVariableTableAttribute(Clazz clazz, Method method, CodeAttribute codeAttribute, LocalVariableTableAttribute localVariableTableAttribute)
         {
             // Write the local variables.
-            dataOutput.writeShort(localVariableTableAttribute.u2localVariableTableLength);
+            dataOutput.writeUnsignedShort(localVariableTableAttribute.u2localVariableTableLength);
 
             localVariableTableAttribute.localVariablesAccept(clazz, method, codeAttribute, this);
         }
@@ -454,7 +454,7 @@ implements   ClassVisitor,
         public void visitLocalVariableTypeTableAttribute(Clazz clazz, Method method, CodeAttribute codeAttribute, LocalVariableTypeTableAttribute localVariableTypeTableAttribute)
         {
             // Write the local variable types.
-            dataOutput.writeShort(localVariableTypeTableAttribute.u2localVariableTypeTableLength);
+            dataOutput.writeUnsignedShort(localVariableTypeTableAttribute.u2localVariableTypeTableLength);
 
             localVariableTypeTableAttribute.localVariablesAccept(clazz, method, codeAttribute, this);
         }
@@ -463,7 +463,7 @@ implements   ClassVisitor,
         public void visitAnyAnnotationsAttribute(Clazz clazz, AnnotationsAttribute annotationsAttribute)
         {
             // Write the annotations.
-            dataOutput.writeShort(annotationsAttribute.u2annotationsCount);
+            dataOutput.writeUnsignedShort(annotationsAttribute.u2annotationsCount);
 
             annotationsAttribute.annotationsAccept(clazz, this);
         }
@@ -480,7 +480,7 @@ implements   ClassVisitor,
                 int          u2annotationsCount = parameterAnnotationsAttribute.u2parameterAnnotationsCount[parameterIndex];
                 Annotation[] annotations        = parameterAnnotationsAttribute.parameterAnnotations[parameterIndex];
 
-                dataOutput.writeShort(u2annotationsCount);
+                dataOutput.writeUnsignedShort(u2annotationsCount);
 
                 for (int index = 0; index < u2annotationsCount; index++)
                 {
@@ -494,7 +494,7 @@ implements   ClassVisitor,
         public void visitAnyTypeAnnotationsAttribute(Clazz clazz, TypeAnnotationsAttribute typeAnnotationsAttribute)
         {
             // Write the type annotations.
-            dataOutput.writeShort(typeAnnotationsAttribute.u2annotationsCount);
+            dataOutput.writeUnsignedShort(typeAnnotationsAttribute.u2annotationsCount);
 
             typeAnnotationsAttribute.typeAnnotationsAccept(clazz, this);
         }
@@ -511,14 +511,14 @@ implements   ClassVisitor,
 
         public void visitBootstrapMethodInfo(Clazz clazz, BootstrapMethodInfo bootstrapMethodInfo)
         {
-            dataOutput.writeShort(bootstrapMethodInfo.u2methodHandleIndex);
+            dataOutput.writeUnsignedShort(bootstrapMethodInfo.u2methodHandleIndex);
 
             // Write the bootstrap method arguments.
-            dataOutput.writeShort(bootstrapMethodInfo.u2methodArgumentCount);
+            dataOutput.writeUnsignedShort(bootstrapMethodInfo.u2methodArgumentCount);
 
             for (int index = 0; index < bootstrapMethodInfo.u2methodArgumentCount; index++)
             {
-                dataOutput.writeShort(bootstrapMethodInfo.u2methodArguments[index]);
+                dataOutput.writeUnsignedShort(bootstrapMethodInfo.u2methodArguments[index]);
             }
         }
 
@@ -527,10 +527,10 @@ implements   ClassVisitor,
 
         public void visitInnerClassesInfo(Clazz clazz, InnerClassesInfo innerClassesInfo)
         {
-            dataOutput.writeShort(innerClassesInfo.u2innerClassIndex);
-            dataOutput.writeShort(innerClassesInfo.u2outerClassIndex);
-            dataOutput.writeShort(innerClassesInfo.u2innerNameIndex);
-            dataOutput.writeShort(innerClassesInfo.u2innerClassAccessFlags);
+            dataOutput.writeUnsignedShort(innerClassesInfo.u2innerClassIndex);
+            dataOutput.writeUnsignedShort(innerClassesInfo.u2outerClassIndex);
+            dataOutput.writeUnsignedShort(innerClassesInfo.u2innerNameIndex);
+            dataOutput.writeUnsignedShort(innerClassesInfo.u2innerClassAccessFlags);
         }
 
 
@@ -538,10 +538,10 @@ implements   ClassVisitor,
 
         public void visitExceptionInfo(Clazz clazz, Method method, CodeAttribute codeAttribute, ExceptionInfo exceptionInfo)
         {
-            dataOutput.writeShort(exceptionInfo.u2startPC);
-            dataOutput.writeShort(exceptionInfo.u2endPC);
-            dataOutput.writeShort(exceptionInfo.u2handlerPC);
-            dataOutput.writeShort(exceptionInfo.u2catchType);
+            dataOutput.writeUnsignedShort(exceptionInfo.u2startPC);
+            dataOutput.writeUnsignedShort(exceptionInfo.u2endPC);
+            dataOutput.writeUnsignedShort(exceptionInfo.u2handlerPC);
+            dataOutput.writeUnsignedShort(exceptionInfo.u2catchType);
         }
 
 
@@ -561,8 +561,8 @@ implements   ClassVisitor,
 
         public void visitLineNumberInfo(Clazz clazz, Method method, CodeAttribute codeAttribute, LineNumberInfo lineNumberInfo)
         {
-            dataOutput.writeShort(lineNumberInfo.u2startPC);
-            dataOutput.writeShort(lineNumberInfo.u2lineNumber);
+            dataOutput.writeUnsignedShort(lineNumberInfo.u2startPC);
+            dataOutput.writeUnsignedShort(lineNumberInfo.u2lineNumber);
         }
 
 
@@ -570,8 +570,8 @@ implements   ClassVisitor,
 
         public void visitParameterInfo(Clazz clazz, Method method, int parameterIndex, ParameterInfo parameterInfo)
         {
-            dataOutput.writeShort(parameterInfo.u2nameIndex);
-            dataOutput.writeShort(parameterInfo.u2accessFlags);
+            dataOutput.writeUnsignedShort(parameterInfo.u2nameIndex);
+            dataOutput.writeUnsignedShort(parameterInfo.u2accessFlags);
         }
 
 
@@ -579,11 +579,11 @@ implements   ClassVisitor,
 
         public void visitLocalVariableInfo(Clazz clazz, Method method, CodeAttribute codeAttribute, LocalVariableInfo localVariableInfo)
         {
-            dataOutput.writeShort(localVariableInfo.u2startPC);
-            dataOutput.writeShort(localVariableInfo.u2length);
-            dataOutput.writeShort(localVariableInfo.u2nameIndex);
-            dataOutput.writeShort(localVariableInfo.u2descriptorIndex);
-            dataOutput.writeShort(localVariableInfo.u2index);
+            dataOutput.writeUnsignedShort(localVariableInfo.u2startPC);
+            dataOutput.writeUnsignedShort(localVariableInfo.u2length);
+            dataOutput.writeUnsignedShort(localVariableInfo.u2nameIndex);
+            dataOutput.writeUnsignedShort(localVariableInfo.u2descriptorIndex);
+            dataOutput.writeUnsignedShort(localVariableInfo.u2index);
         }
 
 
@@ -591,11 +591,11 @@ implements   ClassVisitor,
 
         public void visitLocalVariableTypeInfo(Clazz clazz, Method method, CodeAttribute codeAttribute, LocalVariableTypeInfo localVariableTypeInfo)
         {
-            dataOutput.writeShort(localVariableTypeInfo.u2startPC);
-            dataOutput.writeShort(localVariableTypeInfo.u2length);
-            dataOutput.writeShort(localVariableTypeInfo.u2nameIndex);
-            dataOutput.writeShort(localVariableTypeInfo.u2signatureIndex);
-            dataOutput.writeShort(localVariableTypeInfo.u2index);
+            dataOutput.writeUnsignedShort(localVariableTypeInfo.u2startPC);
+            dataOutput.writeUnsignedShort(localVariableTypeInfo.u2length);
+            dataOutput.writeUnsignedShort(localVariableTypeInfo.u2nameIndex);
+            dataOutput.writeUnsignedShort(localVariableTypeInfo.u2signatureIndex);
+            dataOutput.writeUnsignedShort(localVariableTypeInfo.u2index);
         }
 
 
@@ -604,10 +604,10 @@ implements   ClassVisitor,
         public void visitAnnotation(Clazz clazz, Annotation annotation)
         {
             // Write the annotation type.
-            dataOutput.writeShort(annotation.u2typeIndex);
+            dataOutput.writeUnsignedShort(annotation.u2typeIndex);
 
             // Write the element value pairs.
-            dataOutput.writeShort(annotation.u2elementValuesCount);
+            dataOutput.writeUnsignedShort(annotation.u2elementValuesCount);
 
             annotation.elementValuesAccept(clazz, this);
         }
@@ -642,7 +642,7 @@ implements   ClassVisitor,
 
         public void visitSuperTypeTargetInfo(Clazz clazz, TypeAnnotation typeAnnotation, SuperTypeTargetInfo superTypeTargetInfo)
         {
-            dataOutput.writeShort(superTypeTargetInfo.u2superTypeIndex);
+            dataOutput.writeUnsignedShort(superTypeTargetInfo.u2superTypeIndex);
         }
 
 
@@ -666,14 +666,14 @@ implements   ClassVisitor,
 
         public void visitThrowsTargetInfo(Clazz clazz, Method method, TypeAnnotation typeAnnotation, ThrowsTargetInfo throwsTargetInfo)
         {
-            dataOutput.writeShort(throwsTargetInfo.u2throwsTypeIndex);
+            dataOutput.writeUnsignedShort(throwsTargetInfo.u2throwsTypeIndex);
         }
 
 
         public void visitLocalVariableTargetInfo(Clazz clazz, Method method, CodeAttribute codeAttribute, TypeAnnotation typeAnnotation, LocalVariableTargetInfo localVariableTargetInfo)
         {
             // Write the local variable target elements.
-            dataOutput.writeShort(localVariableTargetInfo.u2tableLength);
+            dataOutput.writeUnsignedShort(localVariableTargetInfo.u2tableLength);
 
             localVariableTargetInfo.targetElementsAccept(clazz, method, codeAttribute, typeAnnotation, this);
         }
@@ -681,19 +681,19 @@ implements   ClassVisitor,
 
         public void visitCatchTargetInfo(Clazz clazz, Method method, CodeAttribute codeAttribute, TypeAnnotation typeAnnotation, CatchTargetInfo catchTargetInfo)
         {
-            dataOutput.writeShort(catchTargetInfo.u2exceptionTableIndex);
+            dataOutput.writeUnsignedShort(catchTargetInfo.u2exceptionTableIndex);
         }
 
 
         public void visitOffsetTargetInfo(Clazz clazz, Method method, CodeAttribute codeAttribute, TypeAnnotation typeAnnotation, OffsetTargetInfo offsetTargetInfo)
         {
-            dataOutput.writeShort(offsetTargetInfo.u2offset);
+            dataOutput.writeUnsignedShort(offsetTargetInfo.u2offset);
         }
 
 
         public void visitTypeArgumentTargetInfo(Clazz clazz, Method method, CodeAttribute codeAttribute, TypeAnnotation typeAnnotation, TypeArgumentTargetInfo typeArgumentTargetInfo)
         {
-            dataOutput.writeShort(typeArgumentTargetInfo.u2offset);
+            dataOutput.writeUnsignedShort(typeArgumentTargetInfo.u2offset);
             dataOutput.writeByte(typeArgumentTargetInfo.u1typeArgumentIndex);
         }
 
@@ -711,9 +711,9 @@ implements   ClassVisitor,
 
         public void visitLocalVariableTargetElement(Clazz clazz, Method method, CodeAttribute codeAttribute, TypeAnnotation typeAnnotation, LocalVariableTargetInfo localVariableTargetInfo, LocalVariableTargetElement localVariableTargetElement)
         {
-            dataOutput.writeShort(localVariableTargetElement.u2startPC);
-            dataOutput.writeShort(localVariableTargetElement.u2length);
-            dataOutput.writeShort(localVariableTargetElement.u2index);
+            dataOutput.writeUnsignedShort(localVariableTargetElement.u2startPC);
+            dataOutput.writeUnsignedShort(localVariableTargetElement.u2length);
+            dataOutput.writeUnsignedShort(localVariableTargetElement.u2index);
         }
 
 
@@ -725,7 +725,7 @@ implements   ClassVisitor,
             int u2elementNameIndex = elementValue.u2elementNameIndex;
             if (u2elementNameIndex != 0)
             {
-                dataOutput.writeShort(u2elementNameIndex);
+                dataOutput.writeUnsignedShort(u2elementNameIndex);
             }
 
             // Write the tag.
@@ -746,7 +746,7 @@ implements   ClassVisitor,
         {
             if (sameZeroFrame.getTag() == StackMapFrame.SAME_ZERO_FRAME_EXTENDED)
             {
-                dataOutput.writeShort(sameZeroFrame.u2offsetDelta);
+                dataOutput.writeUnsignedShort(sameZeroFrame.u2offsetDelta);
             }
         }
 
@@ -755,7 +755,7 @@ implements   ClassVisitor,
         {
             if (sameOneFrame.getTag() == StackMapFrame.SAME_ONE_FRAME_EXTENDED)
             {
-                dataOutput.writeShort(sameOneFrame.u2offsetDelta);
+                dataOutput.writeUnsignedShort(sameOneFrame.u2offsetDelta);
             }
 
             // Write the verification type of the stack entry.
@@ -765,13 +765,13 @@ implements   ClassVisitor,
 
         public void visitLessZeroFrame(Clazz clazz, Method method, CodeAttribute codeAttribute, int offset, LessZeroFrame lessZeroFrame)
         {
-            dataOutput.writeShort(lessZeroFrame.u2offsetDelta);
+            dataOutput.writeUnsignedShort(lessZeroFrame.u2offsetDelta);
         }
 
 
         public void visitMoreZeroFrame(Clazz clazz, Method method, CodeAttribute codeAttribute, int offset, MoreZeroFrame moreZeroFrame)
         {
-            dataOutput.writeShort(moreZeroFrame.u2offsetDelta);
+            dataOutput.writeUnsignedShort(moreZeroFrame.u2offsetDelta);
 
             // Write the verification types of the additional local variables.
             moreZeroFrame.additionalVariablesAccept(clazz, method, codeAttribute, offset, this);
@@ -780,14 +780,14 @@ implements   ClassVisitor,
 
         public void visitFullFrame(Clazz clazz, Method method, CodeAttribute codeAttribute, int offset, FullFrame fullFrame)
         {
-            dataOutput.writeShort(fullFrame.u2offsetDelta);
+            dataOutput.writeUnsignedShort(fullFrame.u2offsetDelta);
 
             // Write the verification types of the local variables.
-            dataOutput.writeShort(fullFrame.variablesCount);
+            dataOutput.writeUnsignedShort(fullFrame.variablesCount);
             fullFrame.variablesAccept(clazz, method, codeAttribute, offset, this);
 
             // Write the verification types of the stack entries.
-            dataOutput.writeShort(fullFrame.stackCount);
+            dataOutput.writeUnsignedShort(fullFrame.stackCount);
             fullFrame.stackAccept(clazz, method, codeAttribute, offset, this);
         }
 
@@ -819,13 +819,13 @@ implements   ClassVisitor,
 
         public void visitObjectType(Clazz clazz, Method method, CodeAttribute codeAttribute, int offset, ObjectType objectType)
         {
-            dataOutput.writeShort(objectType.u2classIndex);
+            dataOutput.writeUnsignedShort(objectType.u2classIndex);
         }
 
 
         public void visitUninitializedType(Clazz clazz, Method method, CodeAttribute codeAttribute, int offset, UninitializedType uninitializedType)
         {
-            dataOutput.writeShort(uninitializedType.u2newInstructionOffset);
+            dataOutput.writeUnsignedShort(uninitializedType.u2newInstructionOffset);
         }
     }
 
@@ -838,20 +838,20 @@ implements   ClassVisitor,
 
         public void visitConstantElementValue(Clazz clazz, Annotation annotation, ConstantElementValue constantElementValue)
         {
-            dataOutput.writeShort(constantElementValue.u2constantValueIndex);
+            dataOutput.writeUnsignedShort(constantElementValue.u2constantValueIndex);
         }
 
 
         public void visitEnumConstantElementValue(Clazz clazz, Annotation annotation, EnumConstantElementValue enumConstantElementValue)
         {
-            dataOutput.writeShort(enumConstantElementValue.u2typeNameIndex);
-            dataOutput.writeShort(enumConstantElementValue.u2constantNameIndex);
+            dataOutput.writeUnsignedShort(enumConstantElementValue.u2typeNameIndex);
+            dataOutput.writeUnsignedShort(enumConstantElementValue.u2constantNameIndex);
         }
 
 
         public void visitClassElementValue(Clazz clazz, Annotation annotation, ClassElementValue classElementValue)
         {
-            dataOutput.writeShort(classElementValue.u2classInfoIndex);
+            dataOutput.writeUnsignedShort(classElementValue.u2classInfoIndex);
         }
 
 
@@ -865,7 +865,7 @@ implements   ClassVisitor,
         public void visitArrayElementValue(Clazz clazz, Annotation annotation, ArrayElementValue arrayElementValue)
         {
             // Write the element values.
-            dataOutput.writeShort(arrayElementValue.u2elementValuesCount);
+            dataOutput.writeUnsignedShort(arrayElementValue.u2elementValuesCount);
 
             arrayElementValue.elementValuesAccept(clazz, annotation, attributeBodyWriter);
         }
